﻿using Kivii.Configuration;
using Kivii.EmailPools.Entities;
using Kivii.Linq;
using Kivii.Web;
using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Net.Mail;
using System.Text;
using System.Threading.Tasks;

namespace Kivii.EmailPools.Transforms
{
    [Api(Description = "邮箱池创建")]
    //[RequiresAnyRole(MemberRoles.Everyone)]
    public class EmailPoolCreate : RestfulExecutionGeneric<EmailPool>
    {
        public EmailPool Item { get; set; }
        public List<EmailPool> Items { get; set; }

        public override object OnExecutionGeneric<G>(IRequest req, IResponse res)
        {
            (Item == null && Items.IsNullOrEmpty()).ThrowIfTrue("Need Item Or Items!");

            if (Items.IsNullOrEmpty()) Items = new List<EmailPool>();
            if (Item != null) Items.Add(Item);
            Console.WriteLine("-----------------------EmailCreateStart---------------------------------");
            var conn = KiviiContext.GetOpenedDbConnection<EmailPool<G>>();
            var rtns = new RestfulCreateResponse<EmailPool<G>>();
            rtns.Results = new List<EmailPool<G>>();

            foreach (var item in Items)
            {
                try
                {
                    //try
                    //{
                    //    Console.WriteLine($"-----------------------from:{item.FromAddress}---------------------------------");
                    //    var from = new MailAddress(item.FromAddress, item.FromAddress);
                    //}
                    //catch
                    //{
                    //    Console.WriteLine($"-----------------------fromCatch:{item.FromAddress}---------------------------------");
                    //    throw new Exception($"发送地址{item.FromAddress},格式错误");
                    //}
                    try
                    {
                        Console.WriteLine($"-----------------------To:{item.ToAddress}---------------------------------");
                        var to = new MailAddress(item.ToAddress, item.ToName);
                    }
                    catch
                    {
                        Console.WriteLine($"-----------------------ToCatch:{item.ToAddress}---------------------------------");
                        throw new Exception($"收件地址{item.ToAddress},格式错误");
                    }
                    EmailPool<G> emailPool = new EmailPool<G>();
                    emailPool.PopulateWith(item);
                    conn.Insert(emailPool);
                    Console.WriteLine("-----------------------EmailCreateEnd---------------------------------");
                    rtns.Results.Add(emailPool);
                }
                catch
                {
                    continue;
                }
            }
            return rtns;
        }
    }

    [Api(Description = "邮箱池更新")]
    //[RequiresAnyRole(MemberRoles.Everyone)]
    public class EmailPoolUpdate : RestfulExecutionGeneric<EmailPool>
    {
        public EmailPool Item { get; set; }

        public List<EmailPool> Items { get; set; }

        public override object OnExecutionGeneric<G>(IRequest req, IResponse res)
        {
            (Item == null && Items.IsNullOrEmpty()).ThrowIfTrue("Need Item Or Items!");

            if (Items.IsNullOrEmpty()) Items = new List<EmailPool>();
            if (Item != null) Items.Add(Item);

            var conn = KiviiContext.GetOpenedDbConnection<EmailPool<G>>();

            var emailPools = conn.SelectByIds<EmailPool<G>>(Items.ConvertAll(o => o.Kvid));
            emailPools.ThrowIfNullOrEmpty("未找到要更新的数据!");
            if (emailPools.Exists(o => o.IsSended))
            {
                throw new Exception("存在已发送的数据,无法更新数据!");
            }

            var rtns = new RestfulUpdateResponse<EmailPool<G>>();
            rtns.Results = new List<EmailPool<G>>();

            foreach (var item in Items)
            {
                var emailPool = emailPools.FirstOrDefault(o => o.Kvid == item.Kvid);
                if(emailPool != null)
                {
                    if (item.OnlyPropertiesIsExist(o => o.ToAddress))
                    {
                        try
                        {
                            var to = new MailAddress(item.ToAddress, item.ToName);
                        }
                        catch
                        {
                            throw new Exception($"收件地址{item.ToAddress},格式错误");
                        }
                        emailPool.ToAddress = item.ToAddress;
                        emailPool.AddOnlyProperties(o => o.ToAddress);
                        conn.UpdateOnly(emailPool);
                        rtns.Results.Add(emailPool);
                    }
                }
            }
            return rtns;
        }
    }

    [Api(Description = "邮箱池查询")]
    //[RequiresAnyRole(MemberRoles.Everyone)]
    public class EmailPoolQuery : RestfulExecutionGeneric<EmailPool>
    {
        #region QueryArgs
        public virtual int? Skip { get; set; }

        public virtual int? Take { get; set; }

        public virtual string OrderBy { get; set; }

        public string OrderByDesc { get; set; }

        public virtual string Include { get; set; }

        public virtual string Fields { get; set; }

        public string QueryKeys { get; set; }

        public string QueryValues { get; set; }

        #endregion
        public override object OnExecutionGeneric<G>(IRequest req, IResponse res)
        {
            Console.WriteLine("---------------Start:Query----------------");
            var conn = KiviiContext.GetOpenedDbConnection<EmailPool<G>>();
            var dynamicParams = Request.GetRequestParams();
            var autoQuery = Request.TryResolve<IAutoQueryDb>();
            autoQuery.IncludeTotal = true;
            var request = new RestfulQuery<EmailPool<G>>();
            request = request.PopulateWith(this);
            var sqlExpress = autoQuery.CreateQuery(Request, conn, request, dynamicParams);
            var rtns = autoQuery.Execute(Request, conn, request, sqlExpress);
            Console.WriteLine($"---------------Query:{rtns.Total}----------------");
            Console.WriteLine("---------------End:Query----------------");
            return rtns;
        }
    }

    [Api(Description = "未发送邮箱池")]
    //[RequiresAnyRole(MemberRoles.Everyone)]
    public class EmailPoolPreSending : RestfulExecutionGeneric<EmailPool>
    {
        public override object OnExecutionGeneric<G>(IRequest req, IResponse res)
        {
            var conn = KiviiContext.GetOpenedDbConnection<EmailPool<G>>();
            var pools = conn.Select<EmailPool<G>>(o => o.IsSended == false);

            var rtns = new RestfulQueryResponse<EmailPool<G>>();
            rtns.Results = new List<EmailPool<G>>();
            rtns.Results.AddRange(pools);
            rtns.Total = rtns.Results.Count;

            return rtns;
        }
    }

    [Api(Description = "标记发送完成")]
    //[RequiresAnyRole(MemberRoles.Everyone)]
    public class EmailPoolComplete : RestfulExecutionGeneric<EmailPool>
    {
        public List<Guid> Kvids { get; set; }
        public string OperatorName { get; set; }
        public Guid OperatorKvid { get; set; }

        public override object OnExecutionGeneric<G>(IRequest req, IResponse res)
        {
            Kvids.ThrowIfNullOrEmpty("Need Kvids");

            var conn = KiviiContext.GetOpenedDbConnection<EmailPool<G>>();
            var pools = conn.SelectByIds<EmailPool<G>>(Kvids);
            pools.ThrowIfNullOrEmpty($"EmailPool Not Found:Kvid={Kvids}");

            var rtns = new RestfulExecutionResponse<EmailPool<G>>();
            rtns.Results = new List<EmailPool<G>>();

            foreach (var item in pools)
            {
                item.IsSended = true;
                item.AddOnlyProperties(o => o.IsSended);
                item.OperatorName = OperatorName;
                item.AddOnlyProperties(o => o.OperatorName);
                item.OperatorKvid = OperatorKvid;
                item.AddOnlyProperties(o => o.OperatorKvid);
                //item.Category = "发送成功";
                //item.AddOnlyProperties(o => o.Category);
                //item.Remark = "";
                //item.AddOnlyProperties(o => o.Remark);
                conn.UpdateOnly(item);
                rtns.Results.Add(item);

                var log = new EntityLog<G>();
                log.OwnerKvid = item.OwnerKvid;
                log.BizId = item.OwnerName;
                log.BizKvid = item.OwnerKvid;
                log.BizType = typeof(G).FullName;
                log.Type = "电子邮箱";
                log.Title = DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss");
                log.Summary = "电子邮箱发送完成";
                log.Remark = $"[发送成功] 已成功将{item.OwnerName} 发送至客户邮箱({item.ToAddress}). 时间:{DateTime.Now:yyyy-MM-dd HH:mm:ss}!";
                conn.Insert(log);
            }

            return rtns;
        }
    }

    [Api(Description = "标记发送失败")]
    //[RequiresAnyRole(MemberRoles.Everyone)]
    public class EmailPoolFail : RestfulExecutionGeneric<EmailPool>
    {
        public List<EmailPool> Items { get; set; }

        public string OperatorName { get; set; }
        public Guid OperatorKvid { get; set; }

        public override object OnExecutionGeneric<G>(IRequest req, IResponse res)
        {
            Items.ThrowIfNullOrEmpty("Need Kvids");

            var conn = KiviiContext.GetOpenedDbConnection<EmailPool<G>>();
            var pools = conn.SelectByIds<EmailPool<G>>(Items.ConvertAll(p => p.Kvid));
            pools.ThrowIfNullOrEmpty($"EmailPools Not Found!");

            var rtns = new RestfulExecutionResponse<EmailPool<G>>();
            rtns.Results = new List<EmailPool<G>>();

            foreach (var item in pools)
            {
                var current = Items.FirstOrDefault(o => o.Kvid == item.Kvid);
                if (current == null) continue;
                item.IsSended = true;
                item.AddOnlyProperties(o => o.IsSended);
                item.OperatorName = OperatorName;
                item.AddOnlyProperties(o => o.OperatorName);
                item.OperatorKvid = OperatorKvid;
                item.AddOnlyProperties(o => o.OperatorKvid);
                //item.Category = $"发送失败";
                //item.AddOnlyProperties(o => o.Category);
                item.Remark = current.Remark;
                item.AddOnlyProperties(o => o.Remark);
                conn.UpdateOnly(item);
                rtns.Results.Add(item);

                var log = new EntityLog<G>();
                log.OwnerKvid = item.OwnerKvid;
                log.BizId = item.OwnerName;
                log.BizKvid = item.OwnerKvid;
                log.BizType = typeof(G).FullName;
                log.Type = "电子邮箱";
                log.Title = DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss");
                log.Summary = "电子邮箱发送失败";
                log.Remark = $"[发送失败] 发送{item.ToAddress}失败原因:{item.Remark}. 时间:{DateTime.Now:yyyy-MM-dd HH:mm:ss}!";
                conn.Insert(log);
            }

            return rtns;
        }
    }

    [Api(Description = "取消发送")]
    //[RequiresAnyRole(MemberRoles.Everyone)]
    public class EmailPoolCancel : RestfulExecutionGeneric<EmailPool>
    {
        public List<Guid> Kvids { get; set; }
        public bool IsDelete { get; set; } = false;

        public override object OnExecutionGeneric<G>(IRequest req, IResponse res)
        {
            Kvids.ThrowIfNullOrEmpty("Need Kvids");

            var conn = KiviiContext.GetOpenedDbConnection<EmailPool<G>>();
            var pools = conn.SelectByIds<EmailPool<G>>(Kvids);
            pools.ThrowIfNullOrEmpty($"EmailPools Not Found!");

            var rtns = new RestfulExecutionResponse<EmailPool<G>>();
            rtns.Results = new List<EmailPool<G>>();

            foreach (var item in pools)
            {
                if (IsDelete)
                {
                    item.Status = -1;
                    item.AddOnlyProperties(o => o.Status);
                    conn.UpdateOnly(item);
                    rtns.Results.Add(item);
                    continue;
                }
                if (item.IsSended) continue;
                item.IsSended = true;
                item.AddOnlyProperties(o => o.IsSended);
                item.OperatorName = KiviiContext.CurrentMember.FullName;
                item.AddOnlyProperties(o => o.OperatorName);
                item.OperatorKvid = KiviiContext.CurrentMember.Kvid;
                item.AddOnlyProperties(o => o.OperatorKvid);
                //item.Category = $"取消发送";
                //item.AddOnlyProperties(o => o.Category);
                conn.UpdateOnly(item);
                rtns.Results.Add(item);

                var log = new EntityLog<G>();
                log.OwnerKvid = item.OwnerKvid;
                log.BizId = item.OwnerName;
                log.BizKvid = item.OwnerKvid;
                log.BizType = typeof(G).FullName;
                log.Type = "电子邮箱";
                log.Title = DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss");
                log.Summary = "取消电子邮箱发送";
                log.Remark = $"[取消发送] 操作人:{KiviiContext.CurrentMember.FullName}. 时间:{DateTime.Now:yyyy-MM-dd HH:mm:ss}!";
                conn.Insert(log);
            }

            return rtns;
        }
    }
}
